@@ -9,7 +9,7 @@
 *
 * Created   :   14.05.2010
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -137,10 +137,13 @@ class ZPushAdminCLI {
     const COMMAND_RESYNCDEVICE = 6;
     const COMMAND_CLEARLOOP = 7;
     const COMMAND_SHOWLASTSYNC = 8;
+    const COMMAND_RESYNCFOLDER = 9;
+    const COMMAND_FIXSTATES = 10;
 
     static private $command;
     static private $user = false;
     static private $device = false;
+    static private $type = false;
     static private $errormessage;
 
     /**
@@ -164,8 +167,13 @@ class ZPushAdminCLI {
                 "\tremove -d DEVICE\t\t Removes all state data of all users synchronized on device DEVICE\n" .
                 "\tremove -u USER -d DEVICE\t Removes all related state data of device DEVICE of user USER\n" .
                 "\tresync -u USER -d DEVICE\t Resynchronizes all data of device DEVICE of user USER\n" .
+                "\tresync -t TYPE \t\t\t Resynchronizes all folders of type 'email', 'calendar', 'contact', 'task' or 'note' for all devices and users.\n" .
+                "\tresync -t TYPE -u USER \t\t Resynchronizes all folders of type 'email', 'calendar', 'contact', 'task' or 'note' for the user USER.\n" .
+                "\tresync -t TYPE -u USER -d DEVICE Resynchronizes all folders of type 'email', 'calendar', 'contact', 'task' or 'note' for a specified device and user.\n" .
+                "\tresync -t FOLDERID -u USER\t Resynchronize the specified folder id only. The USER should be specified.\n" .
                 "\tclearloop\t\t\t Clears system wide loop detection data\n" .
                 "\tclearloop -d DEVICE -u USER\t Clears all loop detection data of a device DEVICE and an optional user USER\n" .
+                "\tfixstates\t\t\t Checks the states for integrity and fixes potential issues\n" .
                 "\n";
     }
 
@@ -193,7 +201,7 @@ class ZPushAdminCLI {
         if (self::$errormessage)
             return;
 
-        $options = getopt("u:d:a:");
+        $options = getopt("u:d:a:t:");
 
         // get 'user'
         if (isset($options['u']) && !empty($options['u']))
@@ -203,9 +211,9 @@ class ZPushAdminCLI {
 
         // get 'device'
         if (isset($options['d']) && !empty($options['d']))
-            self::$device = trim($options['d']);
+            self::$device = strtolower(trim($options['d']));
         else if (isset($options['device']) && !empty($options['device']))
-            self::$device = trim($options['device']);
+            self::$device = strtolower(trim($options['device']));
 
         // get 'action'
         $action = false;
@@ -214,6 +222,12 @@ class ZPushAdminCLI {
         elseif (isset($options['action']) && !empty($options['action']))
             $action = strtolower(trim($options['action']));
 
+        // get 'type'
+        if (isset($options['t']) && !empty($options['t']))
+            self::$type = strtolower(trim($options['t']));
+        elseif (isset($options['type']) && !empty($options['type']))
+            self::$type = strtolower(trim($options['type']));
+
         // get a command for the requested action
         switch ($action) {
             // list data
@@ -256,10 +270,16 @@ class ZPushAdminCLI {
             case "resynchronize":
             case "re-synchronize":
             case "synchronize":
-                if (self::$user === false || self::$device === false)
-                    self::$errormessage = "Not possible to resynchronize device. Device and user must be specified.";
-                else
-                    self::$command = self::COMMAND_RESYNCDEVICE;
+                // full resync
+                if (self::$type === false) {
+                    if (self::$user === false || self::$device === false)
+                        self::$errormessage = "Not possible to resynchronize device. Device and user must be specified.";
+                    else
+                        self::$command = self::COMMAND_RESYNCDEVICE;
+                }
+                else {
+                    self::$command = self::COMMAND_RESYNCFOLDER;
+                }
                 break;
 
             // clear loop detection data
@@ -268,6 +288,12 @@ class ZPushAdminCLI {
                 self::$command = self::COMMAND_CLEARLOOP;
                 break;
 
+            // clear loop detection data
+            case "fixstates":
+            case "fix":
+                self::$command = self::COMMAND_FIXSTATES;
+                break;
+
 
             default:
                 self::UsageInstructions();
@@ -341,16 +367,34 @@ class ZPushAdminCLI {
                 if (self::$device == false) {
                     echo sprintf("Are you sure you want to re-synchronize all devices of user '%s' [y/N]: ", self::$user);
                     $confirm  =  strtolower(trim(fgets(STDIN)));
-                    if ( !($confirm === 'y' || $confirm === 'yes'))
+                    if ( !($confirm === 'y' || $confirm === 'yes')) {
                         echo "Aborted!\n";
                         exit(1);
+                    }
                 }
                 self::CommandResyncDevices();
                 break;
 
+                case self::COMMAND_RESYNCFOLDER:
+                    if (self::$device == false && self::$user == false) {
+                        echo "Are you sure you want to re-synchronize this folder type of all devices and users [y/N]: ";
+                        $confirm  =  strtolower(trim(fgets(STDIN)));
+                        if ( !($confirm === 'y' || $confirm === 'yes')) {
+                            echo "Aborted!\n";
+                            exit(1);
+                        }
+                    }
+                    self::CommandResyncFolder();
+                    break;
+
             case self::COMMAND_CLEARLOOP:
                 self::CommandClearLoopDetectionData();
                 break;
+
+            case self::COMMAND_FIXSTATES:
+                self::CommandFixStates();
+                break;
+
         }
         echo "\n";
     }
@@ -483,6 +527,43 @@ class ZPushAdminCLI {
         echo sprintf("Resync of device '%s' of user '%s': %s", self::$device, self::$user, ($stat)?'Requested':ZLog::GetLastMessage(LOGLEVEL_ERROR)). "\n";
     }
 
+    /**
+     * Command "Resync folder(s)"
+     * Resyncs a folder type of a specific device/user or of all users
+     *
+     * @return
+     * @access public
+     */
+    static public function CommandResyncFolder() {
+        // if no device is specified, search for all devices of a user. If user is not set, all devices are returned.
+        if (self::$device === false) {
+            $devicelist = ZPushAdmin::ListDevices(self::$user);
+            if (empty($devicelist)) {
+                echo "\tno devices/users found\n";
+                return true;
+            }
+        }
+        else
+            $devicelist = array(self::$device);
+
+        foreach ($devicelist as $deviceId) {
+            $users = ZPushAdmin::ListUsers($deviceId);
+            foreach ($users as $user) {
+                if (self::$user && self::$user != $user)
+                    continue;
+                self::resyncFolder($deviceId, $user, self::$type);
+            }
+        }
+
+    }
+
+    /**
+     * Command to clear the loop detection data
+     * Mobiles may enter loop detection (one-by-one synchring due to timeouts / erros).
+     *
+     * @return
+     * @access public
+     */
     static public function CommandClearLoopDetectionData() {
         $stat = false;
         $stat = ZPushAdmin::ClearLoopDetectionData(self::$user, self::$device);
@@ -496,10 +577,102 @@ class ZPushAdminCLI {
            echo sprintf("Loop detection data of device '%s' of user '%s' removed: %s", self::$device, self::$user, ($stat)?'OK':ZLog::GetLastMessage(LOGLEVEL_ERROR)). "\n";
     }
 
+    /**
+     * Resynchronizes a folder type of a device & user
+     *
+     * @param string    $deviceId       the id of the device
+     * @param string    $user           the user
+     * @param string    $type           the folder type
+     *
+     * @return
+     * @access private
+     */
+    static private function resyncFolder($deviceId, $user, $type) {
+        $device = ZPushAdmin::GetDeviceDetails($deviceId, $user);
+
+        if (! $device instanceof ASDevice) {
+            echo sprintf("Folder resync failed: %s\n", ZLog::GetLastMessage(LOGLEVEL_ERROR));
+            return false;
+        }
+
+        $folders = array();
+        foreach ($device->GetAllFolderIds() as $folderid) {
+            // if  submitting a folderid as type to resync a specific folder.
+            if ($folderid == $type) {
+                printf("Found and resynching requested folderid '%s' on device '%s' of user '%s'\n", $folderid, $deviceId, $user);
+                $folders[] = $folderid;
+                break;
+            }
+
+            if ($device->GetFolderUUID($folderid)) {
+                $foldertype = $device->GetFolderType($folderid);
+                switch($foldertype) {
+                    case SYNC_FOLDER_TYPE_APPOINTMENT:
+                    case SYNC_FOLDER_TYPE_USER_APPOINTMENT:
+                        if ($type == "calendar")
+                            $folders[] = $folderid;
+                        break;
+                    case SYNC_FOLDER_TYPE_CONTACT:
+                    case SYNC_FOLDER_TYPE_USER_CONTACT:
+                        if ($type == "contact")
+                            $folders[] = $folderid;
+                        break;
+                    case SYNC_FOLDER_TYPE_TASK:
+                    case SYNC_FOLDER_TYPE_USER_TASK:
+                        if ($type == "task")
+                            $folders[] = $folderid;
+                        break;
+                    case SYNC_FOLDER_TYPE_NOTE:
+                    case SYNC_FOLDER_TYPE_USER_NOTE:
+                        if ($type == "note")
+                            $folders[] = $folderid;
+                        break;
+                    default:
+                        if ($type == "email")
+                            $folders[] = $folderid;
+                        break;
+                }
+            }
+        }
+
+        $stat = ZPushAdmin::ResyncFolder($user, $deviceId, $folders);
+        echo sprintf("Resync of %d folders of type %s on device '%s' of user '%s': %s\n", count($folders), $type, $deviceId, $user, ($stat)?'Requested':ZLog::GetLastMessage(LOGLEVEL_ERROR));
+    }
+
+    /**
+     * Fixes the states for potential issues
+     *
+     * @return
+     * @access private
+     */
+    static private function CommandFixStates() {
+        echo "Validating and fixing states (this can take some time):\n";
+
+        echo "\tChecking username casings: ";
+        if ($stat = ZPushAdmin::FixStatesDifferentUsernameCases())
+            printf("Processed: %d - Converted: %d - Removed: %d\n", $stat[0], $stat[1], $stat[2]);
+        else
+            echo ZLog::GetLastMessage(LOGLEVEL_ERROR) . "\n";
+
+        // fixes ZP-339
+        echo "\tChecking available devicedata & user linking: ";
+        if ($stat = ZPushAdmin::FixStatesDeviceToUserLinking())
+            printf("Processed: %d - Fixed: %d\n", $stat[0], $stat[1]);
+        else
+            echo ZLog::GetLastMessage(LOGLEVEL_ERROR) . "\n";
+
+        echo "\tChecking for unreferenced (obsolete) state files: ";
+        if (($stat = ZPushAdmin::FixStatesUserToStatesLinking()) !== false)
+            printf("Processed: %d - Deleted: %d\n",  $stat[0], $stat[1]);
+        else
+            echo ZLog::GetLastMessage(LOGLEVEL_ERROR) . "\n";
+    }
+
     /**
      * Prints detailed informations about a device
      *
      * @param string    $deviceId       the id of the device
+     * @param string    $user           the user
      *
      * @return
      * @access private
@@ -507,13 +680,16 @@ class ZPushAdminCLI {
     static private function printDeviceData($deviceId, $user) {
         $device = ZPushAdmin::GetDeviceDetails($deviceId, $user);
 
-        if (! $device instanceof ASDevice)
+        if (! $device instanceof ASDevice) {
+            echo sprintf("Folder resync failed: %s\n", ZLog::GetLastMessage(LOGLEVEL_ERROR));
             return false;
+        }
 
         // Gather some statistics about synchronized folders
         $folders = $device->GetAllFolderIds();
         $synchedFolders = 0;
         $synchedFolderTypes = array();
+        $syncedFoldersInProgress = 0;
         foreach ($folders as $folderid) {
             if ($device->GetFolderUUID($folderid)) {
                 $synchedFolders++;
@@ -542,6 +718,15 @@ class ZPushAdminCLI {
                 if (!isset($synchedFolderTypes[$gentype]))
                     $synchedFolderTypes[$gentype] = 0;
                 $synchedFolderTypes[$gentype]++;
+
+                // set the folder name for all folders which are not fully synchronized yet
+                $fstatus = $device->GetFolderSyncStatus($folderid);
+                if ($fstatus !== false && is_array($fstatus)) {
+                    // TODO would be nice if we could see the real name of the folder, right now we use the folder type as name
+                    $fstatus['name'] = $gentype;
+                    $device->SetFolderSyncStatus($folderid, $fstatus);
+                    $syncedFoldersInProgress++;
+                }
             }
         }
         $folderinfo = "";
@@ -572,7 +757,7 @@ class ZPushAdminCLI {
         if ($device->GetDevicePhoneNumber())
             echo "Device Phone nr:\t". $device->GetDevicePhoneNumber(). "\n";
         if ($device->GetDeviceMobileOperator())
-            echo "Device Operator:\t\t". $device->GetDeviceMobileOperator(). "\n";
+            echo "Device Operator:\t". $device->GetDeviceMobileOperator(). "\n";
         if ($device->GetDeviceEnableOutboundSMS())
             echo "Device Outbound SMS:\t". $device->GetDeviceEnableOutboundSMS(). "\n";
 
@@ -580,8 +765,25 @@ class ZPushAdminCLI {
         echo "First sync:\t\t". strftime("%Y-%m-%d %H:%M", $device->GetFirstSyncTime()) ."\n";
         echo "Last sync:\t\t". ($device->GetLastSyncTime() ? strftime("%Y-%m-%d %H:%M", $device->GetLastSyncTime()) : "never")."\n";
         echo "Total folders:\t\t". count($folders). "\n";
-        echo "Synchronized folders:\t". $synchedFolders . "\n";
+        echo "Synchronized folders:\t". $synchedFolders;
+        if ($syncedFoldersInProgress > 0)
+            echo " (". $syncedFoldersInProgress. " in progress)";
+        echo "\n";
         echo "Synchronized data:\t$folderinfo\n";
+        if ($syncedFoldersInProgress > 0) {
+            echo "Synchronization progress:\n";
+            foreach ($folders as $folderid) {
+                $d = $device->GetFolderSyncStatus($folderid);
+                if ($d) {
+                    $status = "";
+                    if ($d['total'] > 0) {
+                        $percent = round($d['done']*100/$d['total']);
+                        $status = sprintf("Status: %s%d%% (%d/%d)", ($percent < 10)?" ":"", $percent, $d['done'], $d['total']);
+                    }
+                    printf("\tFolder: %s%s Sync: %s    %s\n", $d['name'], str_repeat(" ", 12-strlen($d['name'])), $d['status'], $status);
+                }
+            }
+        }
         echo "Status:\t\t\t";
         switch ($device->GetWipeStatus()) {
             case SYNC_PROVISION_RWSTATUS_OK:
